@@ -1,18 +1,20 @@
-## Copyright (C) 2014-2016 Pantxo Diribarne
+## Copyright (C) 2014-2019 Pantxo Diribarne
 ##
-## This program is free software; you can redistribute it and/or modify it
+## This file is part of Octave.
+##
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
+## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
 ##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## Octave is distributed in the hope that it will be useful, but
+## WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see
-## <http://www.gnu.org/licenses/>.
+## along with Octave; see the file COPYING.  If not, see
+## <https://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {} {@var{retval} =} genpropdoc (@var{OBJNAME}, @var{FILENAME})
@@ -29,10 +31,11 @@
 ## @end deftypefn
 
 function genpropdoc (objname, fname)
-  objnames = {"root", "figure", "axes", "line", ...
-              "text", "image", "patch", "surface", "light", ...
-              "uimenu", "uibuttongroup", "uicontextmenu", "uipanel", ...
-              "uicontrol", "uitoolbar", "uipushtool", "uitoggletool"};
+  objnames = {"root", "figure", "axes", ...
+              "image", "light", "line", "patch", "surface", "text", ...
+              "uibuttongroup", "uicontextmenu", "uicontrol", "uipanel", ...
+              "uimenu", "uipushtool", "uitable", "uitoggletool" "uitoolbar"
+             };
 
   ## Base properties
   base = getstructure ("base");
@@ -87,7 +90,7 @@ function s = getdoc (objname, field, base)
   ##
   ## -"printdefault": a boolean (def. true) that specifies whether the
   ##   default value should be printed.  It is useful for properties
-  ##   like root "screendepth" that default to screen dependant values.
+  ##   like root "screendepth" that default to screen dependent values.
 
   packopt = @(c) strjoin (c, " | ");
   markdef = @(s) ["@{" s "@}"];
@@ -108,6 +111,17 @@ function s = getdoc (objname, field, base)
 __prop__ is unused.";
   doc_unused =  "__prop__ is unused.";
 
+  doc_fontangle = "Control whether the font is italic or normal.";
+  doc_fontsize = "Size of the font used for text rendering.  \
+@xref{XREF__objname__fontunits, , fontunits property}.";
+  doc_fontname = "Name of font used for text rendering.  When setting \
+this property, the text rendering engine will search for a matching \
+font in your system.  If none is found then text is rendered using a \
+default sans serif font (same as the default @qcode{\"*\"} value).";
+  doc_fontunits = "Units used to interpret the @qcode{\"fontsize\"} property.";
+  doc_fontweight = "Control the variant of the base font used for \
+text rendering.";
+
   ## Initialize structure
   if (isfield (base, field))
     s = base.(field);
@@ -123,7 +137,17 @@ __prop__ is unused.";
     switch (field)
       case "beingdeleted"
       case "busyaction"
+        s.doc = "Define how Octave handles the execution of this object's \
+callback properties when it is unable to interrupt another object's \
+executing callback.  This is only relevant when the currently executing \
+callback object has its @code{interruptible} property set to \
+\@qcode{\"off\"}.  The __prop__ property of the interrupting callback object \
+indicates whether the interrupting callback is queued (@qcode{\"queue\"} \
+(default)) or discarded (@qcode{\"cancel\"}).\n\
+@xref{Callbacks, , @w{Callbacks section}}.";
+
       case "buttondownfcn"
+        s.doc = "__fcnmsg__";
         s.valid = valid_fcn;
 
       case "children"
@@ -137,13 +161,13 @@ clipped in its parent axes limits.";
       case "createfcn"
         s.doc = "Callback function executed immediately after __objname__ \
 has been created.  Function is set by using default property on root object, \
-e.g., @code{set (0, \"default__objname__createfcn\", \
-'disp (\"__objname__ created!\")')}.";
+e.g., @code{set (groot, \"default__objname__createfcn\", \
+'disp (\"__objname__ created!\")')}.\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "deletefcn"
         s.doc = "Callback function executed immediately before __objname__ \
-is deleted.";
+is deleted.\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "handlevisibility"
@@ -151,11 +175,36 @@ is deleted.";
 handle is not visible in its parent's \"children\" property.";
 
       case "hittest"
+        s.doc = "Specify whether __objname__ processes mouse events \
+or passes them to ancestors of the object.  When enabled, the object may \
+respond to mouse clicks by evaluating the @qcode{\"buttondownfcn\"}, showing \
+the uicontextmenu, and eventually becoming the root \
+@qcode{\"currentobject\"}.  This property is only relevant when the object \
+can accept mouse clicks which is determined by the @qcode{\"pickableparts\"} \
+property.  @xref{XREF__objname__pickableparts, , @w{pickableparts property}}.";
+
       case "interruptible"
+        s.doc = "Specify whether this object's callback functions may be \
+interrupted by other callbacks.  By default __prop__ is @qcode{\"on\"} \
+and callbacks that make use of @code{drawnow}, @code{figure}, @code{waitfor}, \
+@code{getframe} or @code{pause} functions are eventually interrupted.\n\
+@xref{Callbacks, , @w{Callbacks section}}.";
+
       case "parent"
         s.doc = "Handle of the parent graphics object.";
         s.valid = valid_handle;
 
+      case "pickableparts"
+        s.doc = "Specify whether __objname__ will accept mouse clicks.  \
+By default, __prop__ is @qcode{\"visible\"} and only visible parts of the \
+__objname__ or its children may react to mouse clicks.  When __prop__ is \
+@qcode{\"all\"} both visible and invisible parts (or children) may react to \
+mouse clicks.  When __prop__ is @qcode{\"none\"} mouse clicks on the object \
+are ignored and transmitted to any objects underneath this one.  When an \
+object is configured to accept mouse clicks the @qcode{\"hittest\"} property \
+will determine how they are processed.  \
+@xref{XREF__objname__hittest, , @w{hittest property}}.";
+
       case "selected"
       case "selectionhighlight"
       case "tag"
@@ -193,9 +242,15 @@ not rendered on screen.";
         s.doc = doc_unused;
 
       case "parent"
-        s.doc = "Root figure has no parent graphics object.  __prop__ \
+        s.doc = "Root object has no parent graphics object.  __prop__ \
 is always empty.";
 
+      case "hittest"
+        s.doc = doc_unused;
+
+      case "pickableparts"
+        s.doc = doc_unused;
+
       ## Specific properties
       case "callbackobject"
         s.doc = "Graphics handle of the current object whose callback is executing.";
@@ -282,11 +337,16 @@ are visible in their parents' children list, regardless of the value of their \
       case "clipping"
         s.doc = doc_unused;
 
+      case "pickableparts"
+        s.doc = doc_unused;
+
       ## Specific properties
       case "alphamap"
         s.doc = sprintf (doc_notimpl, "Transparency");
 
       case "closerequestfcn"
+        s.doc = "Function that is executed when a figure is deleted.  \
+@xref{XREFclosereq, , closereq function}.\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "color"
@@ -342,6 +402,9 @@ only if the figure's callback @code{windowbuttonmotionfcn} is defined\n\
         s.doc = "Replace the figure and axes background color with white when printing.";
 
       case "keypressfcn"
+        s.doc = "Callback function executed when a keystroke event \
+happens while the figure has focus. The actual key that was pressed \
+can be retrieved using the second argument 'evt' of the function.  __fcnmsg__";
         s.valid = valid_fcn;
 
       case "keyreleasefcn"
@@ -358,12 +421,13 @@ Lowercase value of the key\n\
 @item Modifier:\n\
 A cell array containing strings representing the modifiers pressed with the \
 key.\n\
-@end table";
+@end table\
+\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "menubar"
-        s.doc = "Control the display of the figure menu bar in the upper \
-left of the figure.";
+        s.doc = "Control the display of the figure menu bar at the top \
+of the figure.";
 
       case "name"
         s.doc = "Name to be displayed in the figure title bar.  The name is \
@@ -376,12 +440,20 @@ property.";
       #  s.doc = "Number of current figure (RO).";
 
       case "nextplot"
+        s.doc = "__prop__ is used by high level plotting functions to \
+decide what to do with axes already present in the figure.  \
+@xref{XREFnewplot, , @w{newplot function}}.";
 
       case "numbertitle"
         s.doc = "Display \"Figure\" followed by the numerical figure handle \
 value in the figure title bar.";
 
       case "outerposition"
+        s.doc = "Specify the position and size of the figure including \
+the top menubar and the bottom status bar.  \
+The four elements of the vector are the coordinates of the lower left corner \
+and width and height of the figure.  \
+@xref{XREFfigureunits, , @w{units property}}.";
         s.valid = valid_4elvec;
 
       case "paperorientation"
@@ -438,17 +510,26 @@ units (ex: @code{\"inches\"}) and @code{\"pixels\"} is dependent on the \
         s.doc = doc_unused;
 
       case "position"
-        s.doc = "Specify the position and size of the figure.  \
+        s.doc = "Specify the position and size of the figure canvas.  \
 The four elements of the vector are the coordinates of the lower left corner \
 and width and height of the figure.  \
 @xref{XREFfigureunits, , @w{units property}}.";
         s.valid = valid_4elvec;
 
       case "renderer"
+        s.doc = "Renderering engine used for printing when @code{renderermode} \
+is \"manual\".  __modemsg__.";
+
       case "renderermode"
+        s.doc = "Control whether the renderering engine used for printing is \
+chosen automatically or specified by the @code{renderer} property.  \
+@xref{XREFprint, , @w{print function}}.";
 
       case "resize"
-        s.doc = doc_unused;
+        s.doc = "Control whether the figure can be resized by dragging the \
+window borders and corners using a mouse.  When __prop__ is @qcode{\"off\"} \
+mouse interactions are disabled but the figure can still be resized by \
+changing its @qcode{\"position\"} property.";
 
       case "resizefcn"
         s.doc = "__prop__ is deprecated.  Use @code{sizechangedfcn} instead.";
@@ -458,13 +539,13 @@ and width and height of the figure.  \
         ## FIXME: docstring explaining what "{normal}|open|alt|extend" mean.
 
       case "sizechangedfcn"
-        s.doc = "Callback triggered when the figure window size is changed.";
+        s.doc = "Callback triggered when the figure window size is changed.\
+\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "toolbar"
-        s.doc = "Control the display of the toolbar along the bottom of the \
-figure window.  When set to @qcode{\"auto\"}, the display is based on the \
-value of the @code{menubar} property.";
+        s.doc = "Control the display of the toolbar (along the bottom of the \
+menubar) and the status bar.  When set to @qcode{\"auto\"}, the display is based on the value of the @code{menubar} property.";
 
       case "units"
         s.doc = "The unit used to compute the @code{position} and \
@@ -485,16 +566,22 @@ value of the @code{menubar} property.";
 @code{windowbuttonmotionfcn}, the mouse callback functions.  These \
 callback functions are called when a mouse button is pressed, dragged, or \
 released respectively.  When these callback functions are executed, the \
-@code{currentpoint} property holds the current coordinates of the cursor.";
+@code{currentpoint} property holds the current coordinates of the cursor.\
+\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "windowkeypressfcn"
+        s.doc = "Function that is executed when a key is pressed and \
+the figure has focus.\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "windowkeyreleasefcn"
+        s.doc = "Function that is executed when a key is released and \
+the figure has focus.\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "windowscrollwheelfcn"
+        s.doc = doc_unused;
         s.valid = valid_fcn;
 
       case "windowstyle"
@@ -521,18 +608,24 @@ Changing modes of a visible figure may cause the figure to close and reopen.";
 
       ## Specific properties
       case "activepositionproperty"
+        s.doc = "Specify which of @qcode{\"position\"} or \
+@qcode{\"outerposition\"} properties takes precedence when axes \
+annotations extent changes.  @xref{XREFaxesposition, , @w{position property}}, \
+and @ref{XREFaxesposition, , @w{outerposition property}}.";
+
       case "alim"
         s.doc = sprintf (doc_notimpl, "Transparency");
 
       case "alimmode"
       case "ambientlightcolor"
-        s.doc = sprintf (doc_notimpl, "Light");
+        s.doc = doc_unused;
 
       case "box"
         s.doc = "Control whether the axes has a surrounding box.";
 
       case "boxstyle"
-        s.doc = doc_unused;
+        s.doc = "For 3-D axes, control whether the @qcode{\"full\"} \
+box is drawn or only the 3 @qcode{\"back\"} axes";
 
       case "cameraposition"
         s.valid = valid_3elvec;
@@ -587,27 +680,30 @@ perpendicular to the screen.";
         s.doc = "Specify the relative height and width of the data \
 displayed in the axes.  Setting @code{dataaspectratio} to \
 @w{@code{[1, 2]}} causes the length of one unit as displayed on the x-axis \
-to be the same as the length of 2 units on the y-axis.  __modemsg__.";
+to be the same as the length of 2 units on the y-axis.  \
+@xref{XREFdaspect, , daspect function}.  __modemsg__.";
         s.valid = valid_3elvec;
 
       case "dataaspectratiomode"
       case "fontangle"
+        s.doc = doc_fontangle;
+
       case "fontname"
-        s.doc = "Name of the font used for axes annotations.";
+        s.doc = doc_fontname;
         s.valid = valid_string;
 
       case "fontsize"
-        s.doc = "Size of the font used for axes annotations.  \
-@xref{XREFaxesfontunits, , @w{fontunits property}}.";
+        s.doc = doc_fontsize;
         s.valid = "scalar";
 
       case "fontunits"
-        s.doc = "Unit used to interpret @code{fontsize} property.";
+        s.doc = doc_fontunits;
 
       case "fontsmoothing"
         s.doc = doc_unused;
 
       case "fontweight"
+        s.doc = doc_fontweight;
 
       case "gridalpha"
         s.doc = sprintf (doc_notimpl, "Transparency");
@@ -624,7 +720,8 @@ to be the same as the length of 2 units on the y-axis.  __modemsg__.";
       case "gridlinestyle"
 
       case "labelfontsizemultiplier"
-        s.doc = doc_unused;
+        s.doc = "Ratio between the x/y/zlabel fontsize and the tick \
+label fontsize";
 
       case "layer"
         s.doc = "Control whether the axes is drawn below child graphics \
@@ -637,6 +734,7 @@ objects (ticks, labels, etc.@: covered by plotted objects) or above.";
         s.doc = doc_unused;
 
       case "linewidth"
+        s.doc = "Width of the main axes lines";
 
       case "minorgridalpha"
         s.doc = sprintf (doc_notimpl, "Transparency");
@@ -656,6 +754,12 @@ objects (ticks, labels, etc.@: covered by plotted objects) or above.";
         s.valid = "scalar in the range (0, 1)";
 
       case "nextplot"
+        s.doc = "__prop__ is used by high level plotting functions to \
+decide what to do with graphics objects already present in the axes.  \
+@xref{XREFnewplot, , @w{newplot function}}.  The state of __prop__ \
+is typically controlled using the @code{hold} function.  \
+@xref{XREFhold, , @w{hold function}}.";
+
       case "outerposition"
         s.doc = "Specify the position of the plot including titles, \
 axes, and legend.  The four elements of the vector are the \
@@ -667,10 +771,10 @@ height to be 0.4 and 0.5 respectively.  \
 @xref{XREFaxesposition, , @w{position property}}.";
         s.valid = valid_4elvec;
 
-      case "pickableparts"
-        s.doc = doc_unused;
-
       case "plotboxaspectratio"
+        s.doc = "@xref{XREFpbaspect, , pbaspect function}.  \
+__modemsg__.";
+
       case "plotboxaspectratiomode"
       case "position"
         s.doc = "Specify the position of the plot excluding titles, \
@@ -691,7 +795,7 @@ height to be 0.4 and 0.5 respectively.  \
 
       case "tickdir"
         s.doc = "Control whether axes tick marks project \"in\" to the plot \
-box or \"out\".";
+box or \"out\".  __modemsg__.";
 
       case "tickdirmode"
 
@@ -705,31 +809,46 @@ length of the tickmarks relative to the longest visible axis.";
         s.valid = valid_2elvec;
 
       case "tightinset"
+        s.doc = "Size of the @code{[left bottom right top]} margins \
+around the axes that enclose labels and title annotations.";
+        s.valid = valid_4elvec;
+        s.printdefault = false;
+
       case "title"
         s.doc = "Graphics handle of the title text object.";
         s.valid = valid_handle;
 
       case "titlefontsizemultiplier"
-        s.doc = doc_unused;
+        s.doc = "Ratio between the title fontsize and the tick \
+label fontsize";
+        s.valid = "positive scalar";
 
       case "titlefontweight"
-        s.doc = doc_unused;
+        s.doc = "Control variant of base font used for the axes title.";
 
       case "units"
+        s.doc = "Units used to interpret the @qcode{\"position\"}, \
+@qcode{\"outerposition\"}, and @qcode{\"tightinset\"} properties.";
+
       case "view"
         s.doc = "Two-element vector @code{[azimuth elevation]} specifying \
 the viewpoint for three-dimensional plots";
         s.valid = valid_2elvec;
 
       case "xaxislocation"
+        s.doc = "Control the x axis location.";
+
       case "xcolor"
-        s.doc = "Color of the x-axis.  @xref{Colors, , colorspec}.";
+        s.doc = "Color of the x-axis.  @xref{Colors, , colorspec}.  \
+__modemsg__.";
         s.valid = packopt ({markdef(valid_color), "@qcode{\"none\"}"});
 
       case "xcolormode"
-        s.doc = doc_unused;
 
       case "xdir"
+        s.doc = "Direction of the x axis: @qcode{\"normal\"} is left \
+to right.";
+
       case "xgrid"
         s.doc = "Control whether major x grid lines are displayed.";
 
@@ -739,7 +858,7 @@ the viewpoint for three-dimensional plots";
 
       case "xlim"
         s.doc = "Two-element vector @code{[xmin xmax]} specifying the limits \
-for the x-axis.  __modemsg__.  @xref{XREFxlim, , @w{xlim function}}.";
+for the x-axis.  __modemsg__.   @xref{XREFxlim, , @w{xlim function}}.";
         s.valid = valid_2elvec;
 
       case "xlimmode"
@@ -763,14 +882,18 @@ for the x-axis.  __modemsg__.  @xref{XREFxlim, , @w{xlim function}}.";
 
       case "xtickmode"
       case "yaxislocation"
+        s.doc = "Control the y-axis location.";
+
       case "ycolor"
         s.doc = "Color of the y-axis.  @xref{Colors, , colorspec}.";
         s.valid = packopt ({markdef(valid_color), "@qcode{\"none\"}"});
 
       case "ycolormode"
-        s.doc = doc_unused;
 
       case "ydir"
+        s.doc = "Direction of the y-axis: @qcode{\"normal\"} is bottom \
+to top.";
+
       case "ygrid"
         s.doc = "Control whether major y grid lines are displayed.";
 
@@ -808,8 +931,6 @@ for the y-axis.  __modemsg__.  @xref{XREFylim, , @w{ylim function}}.";
         s.valid = packopt ({markdef(valid_color), "@qcode{\"none\"}"});
 
       case "zcolormode"
-        s.doc = doc_unused;
-
       case "zdir"
       case "zgrid"
         s.doc = "Control whether major z grid lines are displayed.";
@@ -869,6 +990,10 @@ for the z-axis.  __modemsg__.  @xref{XREFzlim, , @w{zlim function}}.";
       case "linewidth"
         s.doc = "Width of the line object measured in points.";
 
+      case "linejoin"
+        s.doc = "Control the shape of the junction of line segments. \
+This property currently only affects the printed output.";
+
       case "marker"
         s.doc = "Shape of the marker for each data point.  \
 @xref{Marker Styles}.";
@@ -927,7 +1052,8 @@ z data.";
 
       ## Specific properties
       case "backgroundcolor"
-        s.doc = sprintf (doc_notimpl, "Background area");
+        s.doc = "Color of the background area.  \
+@xref{Colors, , colorspec}.";
         s.valid = valid_color;
 
       case "color"
@@ -936,30 +1062,35 @@ z data.";
 
       case "displayname"
       case "edgecolor"
-        s.doc = sprintf (doc_notimpl, "Background area");
+        s.doc = "Color of the outline of the background area.  \
+@xref{Colors, , colorspec}.";
         s.valid = valid_color;
 
       case "editing"
+        s.doc = doc_unused;
 
       case "extent"
+        s.doc = "Vector @code{[x0 y0 width height]} indicating the size \
+and location of the text string.";
+        s.valid = valid_4elvec;
+        s.printdefault = false;
+
       case "fontangle"
-        s.doc = "Flag whether the font is italic or normal.  \
-@code{fontangle} is currently unused.";
+        s.doc = doc_fontangle;
 
       case "fontname"
-        s.doc = "The font used for the text.";
+        s.doc = doc_fontname;
         s.valid = valid_string;
 
       case "fontsize"
-        s.doc = "The font size of the text as measured in \
-@code{fontunits}.";
+        s.doc = doc_fontsize;
         s.valid = "scalar";
 
       case "fontunits"
-        s.doc = "The units used to interpret @code{fontsize} property.";
+        s.doc = doc_fontunits;
 
       case "fontweight"
-        s.doc = "Control variant of base font used: bold, light, normal, etc.";
+        s.doc = doc_fontweight;
 
       case "horizontalalignment"
       case "interpreter"
@@ -968,21 +1099,23 @@ interpreted.\n\
 @xref{XREFinterpreterusage, , @w{Use of the interpreter property}}.";
 
       case "linestyle"
-        s.doc = sprintf (doc_notimpl, "Background area");
+        s.doc = "Style of the outline.  @xref{Line Styles}.";
 
       case "linewidth"
-        s.doc = sprintf (doc_notimpl, "Background area");
+        s.doc = "Width of the outline.";
         s.valid = "scalar";
 
       case "margin"
-        s.doc = sprintf (doc_notimpl, "Background area");
+        s.doc = "Margins between the borders of the background area \
+and the texts.  The value is currently interpreted as pixels, regardless \
+of the @qcode{\"fontunits\"} property.";
         s.valid = "scalar";
 
       case "position"
-        s.doc = "Vector @code{[X0 Y0 Z0]} where X0, Y0 and Z0 indicate the \
+        s.doc = "Vector @code{[X0 Y0 Z0]} where X0, Y0, and Z0 indicate the \
 position of the text anchor as defined by @code{verticalalignment} and \
 @code{horizontalalignment}.";
-        s.valid = valid_4elvec;
+        s.valid = valid_3elvec;
 
       case "rotation"
         s.doc = "The angle of rotation for the displayed text, \
@@ -1021,16 +1154,16 @@ measured in degrees.";
         s.valid = valid_cellstring;
 
       case "xdata"
-        s.doc = "Two-element vector @code{[xmin xmax]} specifying the x \
-coordinates of the first and last columns of the image.\n\
+        s.doc = "Two-element vector @code{[xfirst xlast]} specifying the x \
+coordinates of the centers of the first and last columns of the image.\n\
 \n\
 Setting @code{xdata} to the empty matrix ([]) will restore the default value \
 of @code{[1 columns(image)]}.";
         s.valid = valid_2elvec;
 
       case "ydata"
-        s.doc = "Two-element vector @code{[ymin ymax]} specifying the y \
-coordinates of the first and last rows of the image.\n\
+        s.doc = "Two-element vector @code{[yfirst ylast]} specifying the y \
+coordinates of the centers of the first and last rows of the image.\n\
 \n\
 Setting @code{ydata} to the empty matrix ([]) will restore the default value \
 of @code{[1 rows(image)]}.";
@@ -1089,8 +1222,17 @@ the vertices). @qcode{\"phong\"} is deprecated and has the same effect as \
 @qcode{\"gouraud\"}.";
 
       case "facealpha"
-        s.doc = sprintf (doc_notimpl, "Transparency");
-        s.valid = valid_scalmat;
+        s.doc = "Transparency level of the faces of the surface object.  Only \
+double values are supported at present where a value of 0 means complete \
+transparency and a value of 1 means solid faces without transparency.  Setting \
+the property to @qcode{\"flat\"}, @qcode{\"interp\"} or @qcode{\"texturemap\"} \
+causes the faces to not being rendered.  Additionally, the faces are not sorted \
+from back to front which might lead to unexpected results when rendering \
+layered transparent faces.";
+        s.valid = packopt ({"scalar", ...
+                            "@qcode{\"flat\"}", ...
+                            "@qcode{\"interp\"}", ...
+                            "@qcode{\"texturemap\"}"});
 
       case "facecolor"
       case "facelighting"
@@ -1101,6 +1243,17 @@ of the object are drawn with light and shadow effects.  Supported values are \
 the vertices). @qcode{\"phong\"} is deprecated and has the same effect as \
 @qcode{\"gouraud\"}.";
 
+      case "facenormals"
+        s.doc = "Face normals are used for lighting the edges or faces if the \
+@code{edgelighting} or @code{facelighting} properties are set to \
+@qcode{\"flat\"}.  __modemsg__";
+
+      case "facenormalsmode"
+        s.doc = "If this property is set to @qcode{\"auto\"}, \
+@code{facenormals} are automatically calculated if the @code{edgelighting} or \
+@code{facelighting} property are set to @qcode{\"flat\"} and at least one \
+@code{light} object is present and visible in the same axes.";
+
       case "interpreter"
       case "linestyle"
         s.doc = "@xref{Line Styles}.";
@@ -1125,7 +1278,6 @@ the vertices). @qcode{\"phong\"} is deprecated and has the same effect as \
         s.valid = "scalar";
 
       case "meshstyle"
-      case "normalmode"
       case "specularcolorreflectance"
         s.doc = "Reflectance for specular color. Value between 0.0 (color \
 of underlying face) and 1.0 (color of light source).";
@@ -1142,6 +1294,16 @@ specular reflex) and 1.0 (full specular reflex).";
         s.valid = "scalar";
 
       case "vertexnormals"
+        s.doc = "Vertex normals are used for lighting the edges or faces if \
+the @code{edgelighting} or @code{facelighting} properties are set to \
+@qcode{\"gouraud\"}.  __modemsg__";
+
+      case "vertexnormalsmode"
+        s.doc = "If this property is set to @qcode{\"auto\"}, \
+@code{vertexnormals} are automatically calculated if the @code{edgelighting} \
+or @code{facelighting} property are set to @qcode{\"gouraud\"} and at least \
+one @code{light} object is present and visible in the same axes.";
+
       case "xdata"
         s.valid = "matrix";
 
@@ -1212,8 +1374,16 @@ the vertices). @qcode{\"phong\"} is deprecated and has the same effect as \
 @qcode{\"gouraud\"}.";
 
       case "facealpha"
-        s.doc = sprintf (doc_notimpl, "Transparency");
-        s.valid = valid_scalmat;
+        s.doc = "Transparency level of the faces of the patch object.  Only \
+double values are supported at present where a value of 0 means complete \
+transparency and a value of 1 means solid faces without transparency.  Setting \
+the property to @qcode{\"flat\"} or @qcode{\"interp\"} causes the faces to not \
+being rendered.  Additionally, the faces are not sorted from back to front \
+which might lead to unexpected results when rendering layered transparent \
+faces.";
+        s.valid = packopt ({"scalar", ...
+                            "@qcode{\"flat\"}", ...
+                            "@qcode{\"interp\"}"});
 
       case "facecolor"
         ## Don't provide a default value, and mark colorspec with
@@ -1231,8 +1401,18 @@ of the object are drawn with light and shadow effects. Supported values are \
 the vertices). @qcode{\"phong\"} is deprecated and has the same effect as \
 @qcode{\"gouraud\"}.";
 
+      case "facenormals"
+        s.doc = "Face normals are used for lighting the edges or faces if the \
+@code{edgelighting} or @code{facelighting} properties are set to \
+@qcode{\"flat\"}.  __modemsg__";
+
+      case "facenormalsmode"
+        s.doc = "If this property is set to @qcode{\"auto\"}, \
+@code{facenormals} are automatically calculated if the @code{edgelighting} or \
+@code{facelighting} property are set to @qcode{\"flat\"} and at least one \
+@code{light} object is present and visible in the same axes.";
+
       case "faces"
-      case "xdata"
         s.valid = valid_vecmat;
 
       case "facevertexalphadata"
@@ -1260,7 +1440,6 @@ the vertices). @qcode{\"phong\"} is deprecated and has the same effect as \
         s.doc = "@xref{XREFlinemarkersize, , @w{line markersize property}}.";
         s.valid = "scalar";
 
-      case "normalmode"
       case "specularcolorreflectance"
         s.doc = "Reflectance for specular color.  Value between 0.0 (color \
 of underlying face) and 1.0 (color of light source).";
@@ -1277,6 +1456,16 @@ specular reflex) and 1.0 (full specular reflex).";
         s.valid = "scalar";
 
       case "vertexnormals"
+        s.doc = "Vertex normals are used for lighting the edges or faces if \
+the @code{edgelighting} or @code{facelighting} properties are set to \
+@qcode{\"gouraud\"}.  __modemsg__";
+
+      case "vertexnormalsmode"
+        s.doc = "If this property is set to @qcode{\"auto\"}, \
+@code{vertexnormals} are automatically calculated if the @code{edgelighting} \
+or @code{facelighting} property are set to @qcode{\"gouraud\"} and at least \
+one @code{light} object is present and visible in the same axes.";
+
       case "vertices"
         s.valid = valid_vecmat;
 
@@ -1355,15 +1544,37 @@ point source (@qcode{\"local\"}).";
       case "bordertype"
       case "borderwidth"
       case "fontangle"
+        s.doc = doc_fontangle;
+
       case "fontname"
+        s.doc = doc_fontname;
+        s.valid = valid_string;
+
       case "fontsize"
+        s.doc = doc_fontsize;
+        s.valid = "scalar";
+
       case "fontunits"
+        s.doc = doc_fontunits;
+
       case "fontweight"
+        s.doc = doc_fontweight;
+
       case "foregroundcolor"
       case "highlightcolor"
       case "position"
+
       case "resizefcn"
+        s.doc = "__prop__ is deprecated.  Use @code{sizechangedfcn} instead.";
+        s.valid = valid_fcn;
+
       case "shadowcolor"
+
+      case "sizechangedfcn"
+        s.doc = "Callback triggered when the uipanel size is changed.\
+\n\n__fcnmsg__";
+        s.valid = valid_fcn;
+
       case "title"
       case "titleposition"
       case "units"
@@ -1376,24 +1587,46 @@ point source (@qcode{\"local\"}).";
       ## Overridden shared properties
 
       ## Specific properties
-        case "backgroundcolor"
-        case "bordertype"
-        case "borderwidth"
-        case "fontangle"
-        case "fontname"
-        case "fontsize"
-        case "fontunits"
-        case "fontweight"
-        case "foregroundcolor"
-        case "highlightcolor"
-        case "position"
-        case "resizefcn"
-        case "selectedobject"
-        case "selectionchangedfcn"
-        case "shadowcolor"
-        case "title"
-        case "titleposition"
-        case "units"
+      case "backgroundcolor"
+      case "bordertype"
+      case "borderwidth"
+      case "fontangle"
+        s.doc = doc_fontangle;
+
+      case "fontname"
+        s.doc = doc_fontname;
+        s.valid = valid_string;
+
+      case "fontsize"
+        s.doc = doc_fontsize;
+        s.valid = "scalar";
+
+      case "fontunits"
+        s.doc = doc_fontunits;
+
+      case "fontweight"
+        s.doc = doc_fontweight;
+
+      case "foregroundcolor"
+      case "highlightcolor"
+      case "position"
+
+      case "resizefcn"
+        s.doc = "__prop__ is deprecated.  Use @code{sizechangedfcn} instead.";
+        s.valid = valid_fcn;
+
+      case "selectedobject"
+      case "selectionchangedfcn"
+      case "shadowcolor"
+
+      case "sizechangedfcn"
+        s.doc = "Callback triggered when the uibuttongroup size is changed.\
+\n\n__fcnmsg__";
+        s.valid = valid_fcn;
+
+      case "title"
+      case "titleposition"
+      case "units"
 
     endswitch
 
@@ -1408,11 +1641,26 @@ point source (@qcode{\"local\"}).";
       case "cdata"
       case "enable"
       case "extent"
+        s.valid = valid_4elvec;
+        s.printdefault = false;
+
       case "fontangle"
+        s.doc = doc_fontangle;
+
       case "fontname"
+        s.doc = doc_fontname;
+        s.valid = valid_string;
+
       case "fontsize"
+        s.doc = doc_fontsize;
+        s.valid = "scalar";
+
       case "fontunits"
+        s.doc = doc_fontunits;
+
       case "fontweight"
+        s.doc = doc_fontweight;
+
       case "foregroundcolor"
       case "horizontalalignment"
       case "keypressfcn"
@@ -1430,6 +1678,54 @@ point source (@qcode{\"local\"}).";
 
     endswitch
 
+  ## uitable Properties
+  elseif (strcmp (objname, "uitable"))
+    switch (field)
+      ## Overridden shared properties
+
+      ## Specific properties
+      case "backgroundcolor"
+      case "celleditcallback"
+      case "cellselectioncallback"
+      case "columneditable"
+      case "columnformat"
+      case "columnname"
+      case "columnwidth"
+      case "data"
+      case "enable"
+      case "extent"
+        s.valid = valid_4elvec;
+        s.printdefault = false;
+
+      case "fontangle"
+        s.doc = doc_fontangle;
+
+      case "fontname"
+        s.doc = doc_fontname;
+        s.valid = valid_string;
+
+      case "fontsize"
+        s.doc = doc_fontsize;
+        s.valid = "scalar";
+
+      case "fontunits"
+        s.doc = doc_fontunits;
+
+      case "fontweight"
+        s.doc = doc_fontweight;
+
+      case "foregroundcolor"
+      case "keypressfcn"
+      case "keyreleasefcn"
+      case "position"
+      case "rearrangeablecolumns"
+      case "rowname"
+      case "rowstriping"
+      case "tooltipstring"
+      case "units"
+
+    endswitch
+
   ## uitoolbar properties
   elseif (strcmp (objname, "uitoolbar"))
     switch (field)
@@ -1490,12 +1786,15 @@ function strout = expand_doc (strin, field, objname)
 property to be set to @qcode{\"manual\"}";
   modemsg = sprintf (modemsg, field, field);
   strout = strrep (strout, "__modemsg__", modemsg);
+  fcnmsg = "For information on how to write graphics listener \
+functions see @ref{Callbacks, , @w{Callbacks section}}.";
+  strout = strrep (strout, "__fcnmsg__", fcnmsg);
 endfunction
 
 function s = getstructure (objname, base = [])
   hf = [];
   if (! strcmp (objname, "root"))
-    ## Use an improbable number to avoid ishandle to return true for 1
+    ## Use an improbable number to avoid ishghandle to return true for 1
     hf = figure (2265465, "visible", "off");
   endif
 
@@ -1565,7 +1864,7 @@ function def = getdefault (h, objname, field)
   def = get (h, field);
 
   ## Don't print default values for graphics handles
-  if (isscalar (def) && def != 0 && ishandle (def))
+  if (ishghandle (def) && isscalar (def) && def != 0)
     def = "";
   else
     if (ischar (def))
@@ -1588,9 +1887,9 @@ function def = getdefault (h, objname, field)
         endif
 
         ## Replace texinfo reserved characters
-        def = strrep (str, "{", "@{");
+        def = strrep (str, "@", "@@");  # must occur first
+        def = strrep (def, "{", "@{");
         def = strrep (def, "}", "@}");
-        def = strrep (def, "@", "@@");
 
         def = ["@code{" def "}"];
       else
@@ -1607,8 +1906,15 @@ function str = printdoc (objname, obj)
   fields = sort (fieldnames (obj));
   nf = numel (fields);
 
-  ## File header and begining of properties table
-  str = [warn_autogen() "\n\n@table @asis"];
+  ## File header and beginning of properties table
+  str = warn_autogen ();
+  if (strcmp (objname, "root"))
+    str = sprintf ("%s\n\nProperties of the root graphics object:", str);
+  else
+    str = sprintf ("%s\n\nProperties of @code{%s} objects (@pxref{XREF%s,,%s}):",
+                   str, objname, objname, objname);
+  endif
+  str = sprintf ("%s\n\n@table @asis", str);
 
   for ii = 1:nf
     field = fields{ii};
@@ -1616,7 +1922,7 @@ function str = printdoc (objname, obj)
 
     ## @anchor: cross reference using XREFobjnamefield label
     ## Concept index: call info from octave with 'doc ("objname field")'
-    str = sprintf ("%s@anchor{XREF%s%s}\n@cindex %s %s\n",
+    str = sprintf ("%s@anchor{XREF%s%s}\n@prindex %s %s\n",
                    str, objname, field, objname, field);
 
     ## Item
@@ -1650,23 +1956,23 @@ endfunction
 function str = warn_autogen ()
   str = "@c DO NOT EDIT!  Generated automatically by genpropdoc.m.\n\
 \n\
-@c Copyright (C) 2014-2016 Pantxo Diribarne\n\
+@c Copyright (C) 2014-2019 Pantxo Diribarne\n\
 @c\n\
 @c This file is part of Octave.\n\
 @c\n\
-@c Octave is free software; you can redistribute it and/or modify it\n\
-@c under the terms of the GNU General Public License as published by the\n\
-@c Free Software Foundation; either version 3 of the License, or (at\n\
-@c your option) any later version.\n\
+@c Octave is free software: you can redistribute it and/or modify it\n\
+@c under the terms of the GNU General Public License as published by\n\
+@c the Free Software Foundation, either version 3 of the License, or\n\
+@c (at your option) any later version.\n\
 @c\n\
-@c Octave is distributed in the hope that it will be useful, but WITHOUT\n\
-@c ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or\n\
-@c FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License\n\
-@c for more details.\n\
+@c Octave is distributed in the hope that it will be useful, but\n\
+@c WITHOUT ANY WARRANTY; without even the implied warranty of\n\
+@c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
+@c GNU General Public License for more details.\n\
 @c\n\
 @c You should have received a copy of the GNU General Public License\n\
 @c along with Octave; see the file COPYING.  If not, see\n\
-@c <http://www.gnu.org/licenses/>.";
+@c <https://www.gnu.org/licenses/>.";
 endfunction
 
 function str = print_options (val, default)
