@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2016-2018 John W. Eaton
+Copyright (C) 2016-2019 John W. Eaton
 
 This file is part of Octave.
 
@@ -169,13 +169,6 @@ octave_get_sig_number (const char *signame, int *signum)
 #if defined (SIGINFO)
       *signum = SIGINFO;
       return true;
-#endif
-    }
-  else if (! strcmp (signame, "SIGINT"))
-    {
-#if defined (SIGINT)
-      *signum = SIGINT;
-      return true;
 #endif
     }
   else if (! strcmp (signame, "SIGIOT"))
@@ -546,6 +539,30 @@ octave_restore_signal_mask (void)
   sigprocmask (SIG_SETMASK, &octave_signal_mask, 0);
 }
 
+void *
+octave_alloc_signal_mask (void)
+{
+  return malloc (sizeof (sigset_t));
+}
+
+void
+octave_free_signal_mask (void *mask)
+{
+  free (mask);
+}
+
+void
+octave_get_signal_mask (void *mask)
+{
+  sigprocmask (0, 0, mask);
+}
+
+void
+octave_set_signal_mask (void *mask)
+{
+  sigprocmask (SIG_SETMASK, (sigset_t *) mask, 0);
+}
+
 #if ! defined (__WIN32__)
 static const sigset_t *
 octave_async_signals (void)
@@ -709,13 +726,14 @@ print_sigset (FILE *of, const char *prefix, const sigset_t *sigset)
       if (sigismember (sigset, sig))
         {
           cnt++;
-          fprintf (of, "%ld: %s%d (%s)\n", pthread_self (), prefix, sig,
-                   strsignal (sig));
+          fprintf (of, "%ld: %s%d (%s)\n", (long int) pthread_self (),
+                   prefix, sig, strsignal (sig));
         }
     }
 
   if (cnt == 0)
-    fprintf (of, "%ld: %s<empty signal set>\n", pthread_self (), prefix);
+    fprintf (of, "%ld: %s<empty signal set>\n", (long int) pthread_self (),
+             prefix);
 }
 
 static int
