@@ -115,7 +115,7 @@
 ## renderer is used.  This is equivalent to changing the figure's
 ## @qcode{"Renderer"} property.  When the figure @code{RendererMode} property
 ## is @qcode{"auto"} Octave will use the @qcode{"opengl"} renderer for raster
-## formats (e.g., JPEG) and @qcode{"painters"} for vector formats (e.g., PDF).
+## formats (e.g., JPEG) and @qcode{"painters"} for vector formats (e.g., PDF)@.
 ## Those options are only supported for the "qt" graphics tookit.
 ##
 ## @item -svgconvert
@@ -189,9 +189,9 @@
 ## Octave interpreter from recognizing the embedded comma (',').  For example,
 ## by writing @w{"-S640,480"}.
 ##
-## @item  -tight
-## @itemx -loose
-##   Force a tight or loose bounding box for EPS files.  The default is tight.
+## @item  -loose
+## @itemx -tight
+##   Force a tight or loose bounding box for EPS files.  The default is loose.
 ##
 ## @item -@var{preview}
 ##   Add a preview to EPS files.  Supported formats are:
@@ -1014,22 +1014,36 @@ function cmd = lpr (opts)
 
 endfunction
 
-function cmd = pstoedit (opts, devopt)
+function cmd = pstoedit (opts, devopt, do_svg = true)
 
   if (nargin < 2)
     devopt = opts.devopt;
   endif
 
   if (isempty (opts.pstoedit_binary))
-    error ("print:nopstoedit", "print.m: 'pstoedit' is required for specified output format, but binary is not available in PATH");
+    error ("print:nopstoedit", ...
+           ["print.m: 'pstoedit' is required for specified output format, ", ...
+            "but binary is not available in PATH"]);
   endif
 
   dos_shell = (ispc () && ! isunix ());
-  if (dos_shell)
-    cmd = sprintf ("%s -f %s 2> NUL", opts.pstoedit_binary, devopt);
+
+  if (! do_svg)
+    if (dos_shell)
+      cmd = sprintf ("%s -f %s 2> NUL", opts.pstoedit_binary, devopt);
+    else
+      cmd = sprintf ("%s -f %s 2> /dev/null", opts.pstoedit_binary, devopt);
+      endi;f
+    endif
   else
-    ## FIXME: Is this the right thing to do for DOS?
-    cmd = sprintf ("%s -f %s 2> /dev/null", opts.pstoedit_binary, devopt);
+    cmd = svgconvert (opts, devopt);
+    if (dos_shell)
+      cmd = sprintf ("%s & %s -ssp -f %s %%s 2> NUL", cmd, ...
+                     opts.pstoedit_binary, devopt);
+    else
+      cmd = sprintf ("%s ; %s -ssp -f %s %%s 2> /dev/null", cmd,  ...
+                     opts.pstoedit_binary, devopt);
+    endif
   endif
 
   if (opts.debug)
@@ -1057,7 +1071,7 @@ function cmd = svgconvert (opts, devopt)
       fontdir = __octave_config_info__ ("octfontsdir");
     endif
 
-    cmd = sprintf ("%s - %%s %3.2f %s %d %%s", opts.svgconvert_binary, ...
+    cmd = sprintf ("%s - %%s %3.2f %s %d \"%%s\"", opts.svgconvert_binary, ...
                    get (0, "screenpixelsperinch"), ...
                    fullfile (fontdir, "FreeSans.otf"), 1);
 
